# coding: utf8
from __future__ import absolute_import, division


import unittest as ut
import io
import json

from plwn.readers import nodes as nd
from plwn.bases import RelationInfoBase
from plwn import enums as en

try:
    import xml.etree.cElementTree as et
except ImportError:
    import xml.etree.ElementTree as et


__all__ = (
    'AttributesTest',
    'SynsetGraphTest',
    'LexicalGraphTest',
    'MixedGraphTest',
)


class AttributesTest(ut.TestCase):
    """Test storing attributes in graphml output."""

    _PLWNClass = None  # Override in subclass

    def __assert_attr_syn_definition(self, attrkeys):
        self.assertEqual(
            attrkeys[u'syn_data-definition'],
            {
                u'for': u'node',
                u'attr.name': u'definition',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_syn_is_artificial(self, attrkeys):
        self.assertEqual(
            attrkeys[u'syn_data-is_artificial'],
            {
                u'for': u'node',
                u'attr.name': u'is_artificial',
                u'attr.type': u'boolean',
            },
        )

    def __assert_attr_lex_lemma(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-lemma'],
            {
                u'for': u'node',
                u'attr.name': u'lemma',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_pos(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-pos'],
            {
                u'for': u'node',
                u'attr.name': u'pos',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_variant(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-variant'],
            {
                u'for': u'node',
                u'attr.name': u'variant',
                u'attr.type': u'long',
            },
        )

    def __assert_attr_lex_definition(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-definition'],
            {
                u'for': u'node',
                u'attr.name': u'definition',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_sense_examples(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-sense_examples'],
            {
                u'for': u'node',
                u'attr.name': u'sense_examples',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_sense_examples_sources(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-sense_examples_sources'],
            {
                u'for': u'node',
                u'attr.name': u'sense_examples_sources',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_external_links(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-external_links'],
            {
                u'for': u'node',
                u'attr.name': u'external_links',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_usage_notes(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-usage_notes'],
            {
                u'for': u'node',
                u'attr.name': u'usage_notes',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_domain(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-domain'],
            {
                u'for': u'node',
                u'attr.name': u'domain',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_verb_aspect(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-verb_aspect'],
            {
                u'for': u'node',
                u'attr.name': u'verb_aspect',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_is_emotional(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-is_emotional'],
            {
                u'for': u'node',
                u'attr.name': u'is_emotional',
                u'attr.type': u'boolean',
            },
        )

    def __assert_attr_lex_emotion_markedness(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-emotion_markedness'],
            {
                u'for': u'node',
                u'attr.name': u'emotion_markedness',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_emotion_names(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-emotion_names'],
            {
                u'for': u'node',
                u'attr.name': u'emotion_names',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_emotion_valuations(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-emotion_valuations'],
            {
                u'for': u'node',
                u'attr.name': u'emotion_valuations',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_emotion_example(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-emotion_example'],
            {
                u'for': u'node',
                u'attr.name': u'emotion_example',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_lex_emotion_example_secondary(self, attrkeys):
        self.assertEqual(
            attrkeys[u'lu_data-emotion_example_secondary'],
            {
                u'for': u'node',
                u'attr.name': u'emotion_example_secondary',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_edge_type(self, attrkeys):
        self.assertEqual(
            attrkeys[u'edge-type'],
            {
                u'for': u'edge',
                u'attr.name': u'type',
                u'attr.type': u'string',
            },
        )

    def __assert_attr_edge_name(self, attrkeys):
        self.assertEqual(
            attrkeys[u'edge-name'],
            {
                u'for': u'edge',
                u'attr.name': u'name',
                u'attr.type': u'string',
            },
        )

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiperonimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'deminutywność',
            ),
            nd.make_synset_node(
                id=1,
                definition=u'best friend',
                related=((u'hiperonimia', 2),),
            ),
            nd.make_synset_node(
                id=2,
                definition=u'melk',
                is_artificial=True,
            ),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'pies',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                definition=u'Barks.',
                usage_notes=(u'P', u'S'),
                external_links=(u'http://dogs.com',),
                examples=(u'Ala ma psa',),
                examples_sources=(u'Lies!',),
                domain=en.Domain.zw,
                related=((u'deminutywność', 21),),
                is_emotional=True,
                emotion_markedness=en.EmotionMarkedness.ambiguous,
                # Names and valuations need to be tuples, so their ordering is
                # predictable.
                emotion_names=(en.EmotionName.strach, en.EmotionName.zlosc),
                emotion_valuations=(
                    en.EmotionValuation.piekno,
                    en.EmotionValuation.krzywda,
                ),
                emotion_example_1=u'bim',
                emotion_example_2=u'bom',
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'ssak',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.two,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_synset_attrs_all(self):
        """Include all synset attributes."""
        tree = _write_and_read(
            self.__plwn,
            include_attributes=True,
            skip_artificial_synsets=False,
        )

        # Check data keys
        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 4)
        self.__assert_attr_syn_definition(keys)
        self.__assert_attr_syn_is_artificial(keys)
        # The edges are always included
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        # Check data for the synset
        data = _get_data_values(tree.find(u"./graph/node[@id='1']"))
        self.assertEqual(len(data), 2)
        self.assertEqual(data[u'syn_data-definition'], u'best friend')
        self.assertEqual(data[u'syn_data-is_artificial'], u'false')

    def test_synset_attrs_include(self):
        """Include a select attribute of a synset."""
        tree = _write_and_read(
            self.__plwn,
            included_synset_attributes=(u'definition',),
            skip_artificial_synsets=False,
        )

        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 3)
        self.__assert_attr_syn_definition(keys)
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        data = _get_data_values(tree.find(u"./graph/node[@id='1']"))
        self.assertEqual(len(data), 1)
        self.assertEqual(data[u'syn_data-definition'], u'best friend')

    def test_synset_attrs_exclude(self):
        """Include a select attribute of a synset by excluding other ones."""
        tree = _write_and_read(
            self.__plwn,
            excluded_synset_attributes=(u'definition',),
            skip_artificial_synsets=False,
        )

        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 3)
        self.__assert_attr_syn_is_artificial(keys)
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        data = _get_data_values(tree.find(u"./graph/node[@id='2']"))
        self.assertEqual(len(data), 1)
        self.assertEqual(data[u'syn_data-is_artificial'], u'true')

    def test_lexunit_attrs_all(self):
        """Include all lexical unit attributes."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='lexical_unit',
            include_attributes=True,
            skip_artificial_synsets=False,
        )

        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 18)
        self.__assert_attr_lex_lemma(keys)
        self.__assert_attr_lex_pos(keys)
        self.__assert_attr_lex_variant(keys)
        self.__assert_attr_lex_definition(keys)
        self.__assert_attr_lex_sense_examples(keys)
        self.__assert_attr_lex_sense_examples_sources(keys)
        self.__assert_attr_lex_external_links(keys)
        self.__assert_attr_lex_usage_notes(keys)
        self.__assert_attr_lex_domain(keys)
        self.__assert_attr_lex_verb_aspect(keys)
        self.__assert_attr_lex_is_emotional(keys)
        self.__assert_attr_lex_emotion_markedness(keys)
        self.__assert_attr_lex_emotion_names(keys)
        self.__assert_attr_lex_emotion_valuations(keys)
        self.__assert_attr_lex_emotion_example(keys)
        self.__assert_attr_lex_emotion_example_secondary(keys)
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        data = _get_data_values(tree.find(u"./graph/node[@id='11']"))
        self.assertEqual(len(data), 16)
        self.assertEqual(data[u'lu_data-lemma'], u'pies')
        self.assertEqual(data[u'lu_data-pos'], en.PoS.n.value)
        self.assertEqual(data[u'lu_data-variant'], u'1')
        self.assertEqual(data[u'lu_data-definition'], u'Barks.')
        self.assertEqual(
            json.loads(data[u'lu_data-usage_notes']),
            [u'P', u'S'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-external_links']),
            [u'http://dogs.com'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-sense_examples']),
            [u'Ala ma psa'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-sense_examples_sources']),
            [u'Lies!'],
        )
        self.assertEqual(data[u'lu_data-domain'], en.Domain.zw.value)
        self.assertIsNone(data[u'lu_data-verb_aspect'])
        self.assertEqual(data[u'lu_data-is_emotional'], u'true')
        self.assertEqual(
            data[u'lu_data-emotion_markedness'],
            en.EmotionMarkedness.ambiguous.value,
        )
        self.assertEqual(
            json.loads(data[u'lu_data-emotion_names']),
            [en.EmotionName.strach.value, en.EmotionName.zlosc.value],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-emotion_valuations']),
            [
                en.EmotionValuation.krzywda.value,
                en.EmotionValuation.piekno.value,
            ],
        )
        self.assertEqual(data[u'lu_data-emotion_example'], u'bim')
        self.assertEqual(data[u'lu_data-emotion_example_secondary'], u'bom')

    def test_lexunit_attrs_include(self):
        """Include select attributes of a lexical unit."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='lexical_unit',
            included_lexical_unit_attributes=(u'lemma', u'variant'),
            skip_artificial_synsets=False,
        )

        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 4)
        self.__assert_attr_lex_lemma(keys)
        self.__assert_attr_lex_variant(keys)
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        data = _get_data_values(tree.find(u"./graph/node[@id='11']"))
        self.assertEqual(len(data), 2)
        self.assertEqual(data[u'lu_data-lemma'], u'pies')
        self.assertEqual(data[u'lu_data-variant'], u'1')

    def test_lexunit_attrs_exclude(self):
        """Include select attributes of a lexical unit by excluding others."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='lexical_unit',
            excluded_lexical_unit_attributes=(u'lemma', u'variant'),
            skip_artificial_synsets=False,
        )

        keys = _make_attr_key_dicts(tree)
        self.assertEqual(len(keys), 16)
        self.__assert_attr_lex_pos(keys)
        self.__assert_attr_lex_definition(keys)
        self.__assert_attr_lex_sense_examples(keys)
        self.__assert_attr_lex_sense_examples_sources(keys)
        self.__assert_attr_lex_external_links(keys)
        self.__assert_attr_lex_usage_notes(keys)
        self.__assert_attr_lex_domain(keys)
        self.__assert_attr_lex_verb_aspect(keys)
        self.__assert_attr_lex_is_emotional(keys)
        self.__assert_attr_lex_emotion_markedness(keys)
        self.__assert_attr_lex_emotion_names(keys)
        self.__assert_attr_lex_emotion_valuations(keys)
        self.__assert_attr_lex_emotion_example(keys)
        self.__assert_attr_lex_emotion_example_secondary(keys)
        self.__assert_attr_edge_type(keys)
        self.__assert_attr_edge_name(keys)

        data = _get_data_values(tree.find(u"./graph/node[@id='11']"))
        self.assertEqual(len(data), 14)
        self.assertEqual(data[u'lu_data-pos'], en.PoS.n.value)
        self.assertEqual(data[u'lu_data-definition'], u'Barks.')
        self.assertEqual(
            json.loads(data[u'lu_data-usage_notes']),
            [u'P', u'S'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-external_links']),
            [u'http://dogs.com'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-sense_examples']),
            [u'Ala ma psa'],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-sense_examples_sources']),
            [u'Lies!'],
        )
        self.assertEqual(data[u'lu_data-is_emotional'], u'true')
        self.assertEqual(data[u'lu_data-domain'], en.Domain.zw.value)
        # ElementTree reads empty-value nodes as None
        self.assertIsNone(data[u'lu_data-verb_aspect'])
        self.assertEqual(
            data[u'lu_data-emotion_markedness'],
            en.EmotionMarkedness.ambiguous.value,
        )
        self.assertEqual(
            json.loads(data[u'lu_data-emotion_names']),
            [en.EmotionName.strach.value, en.EmotionName.zlosc.value],
        )
        self.assertEqual(
            json.loads(data[u'lu_data-emotion_valuations']),
            [
                en.EmotionValuation.krzywda.value,
                en.EmotionValuation.piekno.value,
            ],
        )
        self.assertEqual(data[u'lu_data-emotion_example'], u'bim')
        self.assertEqual(data[u'lu_data-emotion_example_secondary'], u'bom')


class SynsetGraphTest(ut.TestCase):
    """Create a graph of synsets and test its edges.

    The last element is artificial so it can be cut off with a parameter.

    ::
        .---.  a   .---.  B/b  .----.
        | 1 |----->| 2 |------>| 3a |
        '---'      '---'       '----'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(kind=en.RelationKind.synset, name=u'a'),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'b',
                parent=u'B',
            ),
            nd.make_synset_node(id=1, related=((u'a', 2),)),
            nd.make_synset_node(id=2, related=((u'B/b', 3),)),
            nd.make_synset_node(id=3, is_artificial=True),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'A',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'B',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'C',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.dk,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_full_graph(self):
        """Include all nodes and edges in the graph."""
        tree = _write_and_read(self.__plwn, skip_artificial_synsets=False)

        # Check nodes
        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'1', u'2', u'3')),
        )

        # Check edges
        edges = _make_edge_dict(tree)

        self.assertEqual(len(edges), 2)
        self.assertEqual(
            edges[u'syn--1--2--a'],
            {
                u'source': u'1',
                u'target': u'2',
                u'type': u'relation',
                u'name': u'a',
            },
        )
        self.assertEqual(
            edges[u'syn--2--3--B/b'],
            {
                u'source': u'2',
                u'target': u'3',
                u'type': u'relation',
                u'name': u'B/b',
            },
        )

    def test_include_relations(self):
        """Include only some of relation edges in the graph."""
        tree = _write_and_read(
            self.__plwn,
            included_synset_relations=(u'a',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'1', u'2')),
        )

        edges = _make_edge_dict(tree)

        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'syn--1--2--a'],
            {
                u'source': u'1',
                u'target': u'2',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_exclude_relations(self):
        """Exclude only some of relation edges in the graph."""
        tree = _write_and_read(
            self.__plwn,
            excluded_synset_relations=(u'a',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'2', u'3')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'syn--2--3--' + RelationInfoBase.format_name(u'B', u'b')],
            {
                u'source': u'2',
                u'target': u'3',
                u'type': u'relation',
                u'name': RelationInfoBase.format_name(u'B', u'b'),
            },
        )

    def test_include_nodes(self):
        """Include only some nodes in the graph."""
        tree = _write_and_read(
            self.__plwn,
            included_synset_nodes=(1, 2),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'1', u'2')),
        )

        edges = _make_edge_dict(tree)

        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'syn--1--2--a'],
            {
                u'source': u'1',
                u'target': u'2',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_exclude_nodes(self):
        """Exclude some nodes from the graph."""
        tree = _write_and_read(
            self.__plwn,
            excluded_synset_nodes=(3,),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'1', u'2')),
        )

        edges = _make_edge_dict(tree)

        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'syn--1--2--a'],
            {
                u'source': u'1',
                u'target': u'2',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_break_node_chain(self):
        """Exclude one node in the middle which should make the graph empty."""
        tree = _write_and_read(
            self.__plwn,
            excluded_synset_nodes=(2,),
            skip_artificial_synsets=False,
        )

        self.assertFalse(_get_node_id_set(tree))
        self.assertFalse(_make_edge_dict(tree))

    def test_prefix_nodes(self):
        """Check if nodes are correctly prefixed when told to."""
        tree = _write_and_read(
            self.__plwn,
            prefix_ids=True,
            skip_artificial_synsets=False,
        )
        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'synset-1', u'synset-2', u'synset-3')),
        )
        self.assertEqual(
            frozenset(_make_edge_dict(tree)),
            frozenset((
                u'syn--synset-1--synset-2--a',
                u'syn--synset-2--synset-3--' +
                RelationInfoBase.format_name(u'B', u'b'),
            )),
        )

    def test_no_artificial(self):
        """Cut off the artificial synset from the rest of the synset graph."""
        tree = _write_and_read(self.__plwn, skip_artificial_synsets=True)

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'1', u'2')),
        )

        edges = _make_edge_dict(tree)

        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'syn--1--2--a'],
            {
                u'source': u'1',
                u'target': u'2',
                u'type': u'relation',
                u'name': u'a',
            },
        )


class LexicalGraphTest(ut.TestCase):
    """Create a graph of lexical units and test its edges.

    The graph is the same as in synset graph test, but with lexical units.
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'a',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'b',
                parent=u'B',
            ),
            nd.make_synset_node(id=1),
            nd.make_synset_node(id=2),
            # On lexical unit level artificial synsets have no meaning
            nd.make_synset_node(id=3, is_artificial=True),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'A',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'a', 21),),
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'B',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((RelationInfoBase.format_name(u'B', u'b'), 31),),
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'C',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.dk,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_full_graph(self):
        """Include all lexical units and relations in the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='lexical_unit',
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'11', u'21', u'31')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 2)
        self.assertEqual(
            edges[u'lu--11--21--a'],
            {
                u'source': u'11',
                u'target': u'21',
                u'type': u'relation',
                u'name': u'a',
            },
        )
        self.assertEqual(
            edges[u'lu--21--31--' + RelationInfoBase.format_name(u'B', u'b')],
            {
                u'source': u'21',
                u'target': u'31',
                u'type': u'relation',
                u'name': RelationInfoBase.format_name(u'B', u'b'),
            },
        )

    def test_include_relations(self):
        """Include only some of the lexical unit relations in the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type=u'lexical_unit',
            included_lexical_unit_relations=(u'a',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'11', u'21')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'lu--11--21--a'],
            {
                u'source': u'11',
                u'target': u'21',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_exclude_relations(self):
        """Exclude some of the relations from the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type=u'lexical_unit',
            excluded_lexical_unit_relations=(u'a',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'21', u'31')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'lu--21--31--' + RelationInfoBase.format_name(u'B', u'b')],
            {
                u'source': u'21',
                u'target': u'31',
                u'type': u'relation',
                u'name': RelationInfoBase.format_name(u'B', u'b'),
            },
        )

    def test_include_nodes(self):
        """Include only some of the lexical unit nodes in the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type=u'lexical_unit',
            included_lexical_unit_nodes=(11, 21,),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'11', u'21')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'lu--11--21--a'],
            {
                u'source': u'11',
                u'target': u'21',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_exclude_nodes(self):
        """Exclude some of the lexical unit nodes in the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type=u'lexical_unit',
            excluded_lexical_unit_nodes=(31,),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((u'11', u'21')),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 1)
        self.assertEqual(
            edges[u'lu--11--21--a'],
            {
                u'source': u'11',
                u'target': u'21',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def test_break_node_chain(self):
        """Exclude one node in the middle which should make the graph empty."""
        tree = _write_and_read(
            self.__plwn,
            excluded_synset_nodes=(2,),
            skip_artificial_synsets=False,
        )

        self.assertFalse(_get_node_id_set(tree))
        self.assertFalse(_make_edge_dict(tree))

    def test_prefix_nodes(self):
        """Make sure node ids are prefixed when told to."""
        tree = _write_and_read(
            self.__plwn,
            graph_type=u'lexical_unit',
            prefix_ids=True,
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
            ))
        )
        self.assertEqual(
            frozenset(_make_edge_dict(tree)),
            frozenset((
                u'lu--lexical_unit-11--lexical_unit-21--a',
                u'lu--lexical_unit-21--lexical_unit-31--' +
                RelationInfoBase.format_name(u'B', u'b'),
            ))
        )


class MixedGraphTest(ut.TestCase):
    """Test how a mixed synset-unit graph behaves.

    ::
                     .--------.-->.-------.
                     | syn-4a |   | lu-41 |
                     '--------'<--'-------'
                        ^ |
                     C/u| |C/d
                        | v
        .-------. A  .-------. B  .-------.
        | syn-1 |--->| syn-2 |--->| syn-3 |
        '-------'    '-------'    '-------'
           ^ |          ^ |          ^ |
           | |          | |          | |
           | v          | v          | v
        .-------. a  .-------. b  .-------.
        | lu-11 |--->| lu-21 |--->| lu-31 |
        '-------'    '-------'    '-------'
    """

    _PLWNClass = None  # Override in subclass

    def __assert_edge_syn_1_2(self, edges):
        self.assertEqual(
            edges[u'syn--synset-1--synset-2--A'],
            {
                u'source': u'synset-1',
                u'target': u'synset-2',
                u'type': u'relation',
                u'name': u'A',
            },
        )

    def __assert_edge_syn_2_3(self, edges):
        self.assertEqual(
            edges[u'syn--synset-2--synset-3--B'],
            {
                u'source': u'synset-2',
                u'target': u'synset-3',
                u'type': u'relation',
                u'name': u'B',
            },
        )

    def __assert_edge_syn_2_4(self, edges):
        self.assertEqual(
            edges[u'syn--synset-2--synset-4--C/u'],
            {
                u'source': u'synset-2',
                u'target': u'synset-4',
                u'type': u'relation',
                u'name': u'C/u',
            },
        )

    def __assert_edge_syn_4_2(self, edges):
        self.assertEqual(
            edges[u'syn--synset-4--synset-2--C/d'],
            {
                u'source': u'synset-4',
                u'target': u'synset-2',
                u'type': u'relation',
                u'name': u'C/d',
            },
        )

    def __assert_edge_lex_11_21(self, edges):
        self.assertEqual(
            edges[u'lu--lexical_unit-11--lexical_unit-21--a'],
            {
                u'source': u'lexical_unit-11',
                u'target': u'lexical_unit-21',
                u'type': u'relation',
                u'name': u'a',
            },
        )

    def __assert_edge_lex_21_31(self, edges):
        self.assertEqual(
            edges[u'lu--lexical_unit-21--lexical_unit-31--b'],
            {
                u'source': u'lexical_unit-21',
                u'target': u'lexical_unit-31',
                u'type': u'relation',
                u'name': u'b',
            },
        )

    def __assert_edge_s2l_1_11(self, edges):
        self.assertEqual(
            edges[u'uns--synset-1--lexical_unit-11--has_unit'],
            {
                u'source': u'synset-1',
                u'target': u'lexical_unit-11',
                u'type': u'unit_and_synset',
                u'name': u'has_unit',
            },
        )

    def __assert_edge_s2l_2_21(self, edges):
        self.assertEqual(
            edges[u'uns--synset-2--lexical_unit-21--has_unit'],
            {
                u'source': u'synset-2',
                u'target': u'lexical_unit-21',
                u'type': u'unit_and_synset',
                u'name': u'has_unit',
            },
        )

    def __assert_edge_s2l_3_31(self, edges):
        self.assertEqual(
            edges[u'uns--synset-3--lexical_unit-31--has_unit'],
            {
                u'source': u'synset-3',
                u'target': u'lexical_unit-31',
                u'type': u'unit_and_synset',
                u'name': u'has_unit',
            },
        )

    def __assert_edge_s2l_4_41(self, edges):
        self.assertEqual(
            edges[u'uns--synset-4--lexical_unit-41--has_unit'],
            {
                u'source': u'synset-4',
                u'target': u'lexical_unit-41',
                u'type': u'unit_and_synset',
                u'name': u'has_unit',
            },
        )

    def __assert_edge_l2s_11_1(self, edges):
        self.assertEqual(
            edges[u'uns--lexical_unit-11--synset-1--in_synset'],
            {
                u'source': u'lexical_unit-11',
                u'target': u'synset-1',
                u'type': u'unit_and_synset',
                u'name': u'in_synset',
            },
        )

    def __assert_edge_l2s_21_2(self, edges):
        self.assertEqual(
            edges[u'uns--lexical_unit-21--synset-2--in_synset'],
            {
                u'source': u'lexical_unit-21',
                u'target': u'synset-2',
                u'type': u'unit_and_synset',
                u'name': u'in_synset',
            },
        )

    def __assert_edge_l2s_31_3(self, edges):
        self.assertEqual(
            edges[u'uns--lexical_unit-31--synset-3--in_synset'],
            {
                u'source': u'lexical_unit-31',
                u'target': u'synset-3',
                u'type': u'unit_and_synset',
                u'name': u'in_synset',
            },
        )

    def __assert_edge_l2s_41_4(self, edges):
        self.assertEqual(
            edges[u'uns--lexical_unit-41--synset-4--in_synset'],
            {
                u'source': u'lexical_unit-41',
                u'target': u'synset-4',
                u'type': u'unit_and_synset',
                u'name': u'in_synset',
            },
        )

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(kind=en.RelationKind.synset, name=u'A'),
            nd.make_relation_type_node(kind=en.RelationKind.synset, name=u'B'),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'u',
                parent=u'C',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'd',
                parent=u'C',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'a',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'b',
            ),
            nd.make_synset_node(id=1, related=((u'A', 2),)),
            nd.make_synset_node(id=2, related=((u'B', 3), (u'C/u', 4))),
            nd.make_synset_node(id=3),
            nd.make_synset_node(
                id=4,
                related=((u'C/d', 2),),
                is_artificial=True,
            ),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'first',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'a', 21),),
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'second',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'b', 31),),
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'third',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'unreal',
                pos=en.PoS.n,
                variant=1,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_full_graph(self):
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 14)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_include_synset_edges(self):
        """Include only certain synset edges in the graph.

        All nodes should remain enact, except for 4 connected by excluded syn
        relations.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            included_synset_relations=(u'B',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 9)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)

    def test_exclude_synset_edges(self):
        """Exclude some synset edges in the graph.

        All nodes should remain enact.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_synset_relations=(u'B',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 13)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_include_lexical_relations(self):
        """Include only some of the lexical relation edges in the graph.

        All nodes should remain.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            included_lexical_unit_relations=(u'b',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 13)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_exclude_lexical_relations(self):
        """Exclude some of the lexical relation edges from the graph.

        All nodes should remain.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_lexical_unit_relations=(u'b',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 13)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_cut_off_nodes(self):
        """Remove both corresponding lexical unit and synset edges.

        This should make a synset and a unit disappear from the graph. They are
        not connected to any other synset or unit, and only have their
        has_unit/in_synset edges.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_synset_relations=(u'B',),
            excluded_lexical_unit_relations=(u'b',),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-4',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 10)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_include_synset_nodes(self):
        """Include only some of the synsets in the graph.

        This should cause corresponding lexical units to be dropped from the
        graph, as well as relation edges.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            included_synset_nodes=(1, 2),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'lexical_unit-11',
                u'lexical_unit-21',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 6)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)

    def test_exclude_synset_nodes(self):
        """Exclude some synsets from the graph.

        Some nodes should drop, like in the include test.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_synset_nodes=(1,),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 10)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_include_lexical_unit_nodes(self):
        """Include only some of lexical unit nodes in the graph.

        Same thing like with including synsets.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            included_lexical_unit_nodes=(11, 21),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'lexical_unit-11',
                u'lexical_unit-21',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 6)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)

    def test_exclude_lexical_unit_nodes(self):
        """Exclude some of lexical unit nodes in the graph.

        Same thing like with excluding synsets.
        """
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_lexical_unit_nodes=(11,),
            skip_artificial_synsets=False,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-2',
                u'synset-3',
                u'synset-4',
                u'lexical_unit-21',
                u'lexical_unit-31',
                u'lexical_unit-41',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 10)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_syn_2_4(edges)
        self.__assert_edge_syn_4_2(edges)
        self.__assert_edge_lex_21_31(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_s2l_4_41(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_l2s_41_4(edges)

    def test_break_graph(self):
        """Remove one of the middle nodes, to empty the whole graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            excluded_synset_nodes=(2,),
            skip_artificial_synsets=False,
        )

        self.assertFalse(_get_node_id_set(tree))
        self.assertFalse(_make_edge_dict(tree))

    def test_no_artificial(self):
        """Omit the artificial synset from the graph."""
        tree = _write_and_read(
            self.__plwn,
            graph_type='mixed',
            skip_artificial_synsets=True,
        )

        self.assertEqual(
            _get_node_id_set(tree),
            frozenset((
                u'synset-1',
                u'synset-2',
                u'synset-3',
                u'lexical_unit-11',
                u'lexical_unit-21',
                u'lexical_unit-31',
            )),
        )

        edges = _make_edge_dict(tree)
        self.assertEqual(len(edges), 10)
        self.__assert_edge_l2s_11_1(edges)
        self.__assert_edge_l2s_21_2(edges)
        self.__assert_edge_l2s_31_3(edges)
        self.__assert_edge_s2l_1_11(edges)
        self.__assert_edge_s2l_2_21(edges)
        self.__assert_edge_s2l_3_31(edges)
        self.__assert_edge_syn_1_2(edges)
        self.__assert_edge_syn_2_3(edges)
        self.__assert_edge_lex_11_21(edges)
        self.__assert_edge_lex_21_31(edges)


def _write_and_read(plwn, **kwargs):
    outfile = io.BytesIO()
    plwn.to_graphml(outfile, **kwargs)
    return et.XML(outfile.getvalue())


def _get_data_values(synset_elem):
    return {
        data_elem.attrib[u'key']: data_elem.text
        for data_elem in synset_elem.findall(u'./data')
    }


def _make_attr_key_dicts(xml_root):
    dicts = {}

    for key_elem in xml_root.findall(u'./key'):
        attrdict = key_elem.attrib.copy()
        attrid = attrdict.pop(u'id')
        dicts[attrid] = attrdict

    return dicts


def _get_node_id_set(xml_root):
    return frozenset(
        elem.attrib[u'id']
        for elem in xml_root.findall(u'./graph/node')
    )


def _make_edge_dict(xml_root):
    return {
        edge_elem.attrib[u'id']: {
            u'source': edge_elem.attrib[u'source'],
            u'target': edge_elem.attrib[u'target'],
            u'type': edge_elem.find(u"./data[@key='edge-type']").text,
            u'name': edge_elem.find(u"./data[@key='edge-name']").text,
        }
        for edge_elem in xml_root.findall(u'./graph/edge')
    }
