# coding: utf8
from __future__ import absolute_import, division


import unittest as ut

from plwn.readers import nodes as nd
from plwn.bases import RelationEdge, RelationInfoBase
from plwn import exceptions as exc, enums as en


__all__ = (
    'ItemSelectingTest',
    'SynsetRelationEdgesTest',
    'SynsetRelationEdgesWithArtificialTest',
    'SynsetRelationEdgesWithArtificialLoopTest',
    'LexicalUnitRelationEdgesTest',
)


class ItemSelectingTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(id=1),
            nd.make_synset_node(id=2, definition=u'tok'),
            nd.make_synset_node(id=3),
            nd.make_synset_node(id=4),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'pies',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'kot',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
                definition=u'kici',
            ),
            nd.make_lexical_unit_node(
                id=22,
                lemma=u'kot',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=2,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'pies',
                pos=en.PoS.n,
                variant=2,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
                emotion_markedness=en.EmotionMarkedness.strong_negative,
                emotion_names=(en.EmotionName.fear, en.EmotionName.disgust),
                emotion_example_1=u'bim',
            ),
            # "Pies" is not a verb, but still, for testing...
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'pies',
                pos=en.PoS.v,
                variant=1,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.ndk,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_synsets_nonexistent(self):
        """Select a synset that doesn't exists (in "synsets" method)."""
        self.assertFalse(tuple(self.__plwn.synsets(u'not')))

    def test_synsets_by_lemma(self):
        """Select several synsets with the same lemmas."""
        self.assertEqual(
            frozenset(self.__plwn.synsets(u'pies')),
            frozenset((
                self.__plwn.synset_by_id(1),
                self.__plwn.synset_by_id(3),
                self.__plwn.synset_by_id(4),
            )),
        )

    def test_synsets_by_pos(self):
        """Select several synsets with the same pos."""
        self.assertEqual(
            frozenset(self.__plwn.synsets(pos=en.PoS.n)),
            frozenset((
                self.__plwn.synset_by_id(1),
                self.__plwn.synset_by_id(2),
                self.__plwn.synset_by_id(3),
            )),
        )

    def test_synsets_by_combination(self):
        """Select several synsets by lemma and pos ."""
        self.assertEqual(
            frozenset(self.__plwn.synsets(lemma=u'pies', pos=en.PoS.n)),
            frozenset((
                self.__plwn.synset_by_id(1),
                self.__plwn.synset_by_id(3),
            )),
        )

    def test_synsets_one(self):
        """Select one synset by "synsets" method."""
        self.assertEqual(
            frozenset(
                self.__plwn.synsets(lemma=u'pies', pos=en.PoS.n, variant=1),
            ),
            frozenset((self.__plwn.synset_by_id(1),)),
        )

    def test_synset_multi_lexunits(self):
        """Select lemma / pos combination.

        That points to multiple lexical units within the same synset.
        """
        self.assertEqual(
            frozenset(self.__plwn.synsets(lemma=u'kot', pos=en.PoS.n)),
            frozenset((self.__plwn.synset_by_id(2),)),
        )

    def test_synsets_all(self):
        """Select all synsets."""
        self.assertEqual(
            frozenset(self.__plwn.synsets()),
            frozenset((
                self.__plwn.synset_by_id(1),
                self.__plwn.synset_by_id(2),
                self.__plwn.synset_by_id(3),
                self.__plwn.synset_by_id(4),
            )),
        )

    def test_synset_any_none(self):
        """Make sure the method doesn't allow ``None`` as arguments."""
        self.assertRaises(
            exc.SynsetNotFound,
            self.__plwn.synset,
            None,
            en.PoS.n,
            1,
        )
        self.assertRaises(
            exc.SynsetNotFound,
            self.__plwn.synset,
            u'kot',
            None,
            1,
        )
        self.assertRaises(
            exc.SynsetNotFound,
            self.__plwn.synset,
            u'kot',
            en.PoS.n,
            None,
        )

    def test_synset_one(self):
        """Select a single synset."""
        sel_one = self.__plwn.synset(lemma=u'kot', pos=en.PoS.n, variant=1)
        sel_id = self.__plwn.synset_by_id(2)

        self.assertEqual(sel_one, sel_id)
        # As this tests both "synset" and "synset_by_id", ensure all properties
        # are correct; those that were set, anyway.
        self.assertEqual(sel_one.id, 2)
        self.assertEqual(sel_one.id, sel_id.id)
        self.assertEqual(sel_one.definition, u'tok')
        self.assertEqual(sel_one.definition, sel_id.definition)
        self.assertFalse(sel_one.is_artificial)
        self.assertEqual(sel_one.is_artificial, sel_id.is_artificial)
        self.assertEqual(
            sel_one.lexical_units,
            (
                self.__plwn.lexical_unit_by_id(21),
                self.__plwn.lexical_unit_by_id(22),
            ),
        )
        self.assertEqual(sel_one.lexical_units, sel_id.lexical_units)

    def test_synset_nonexistent(self):
        """Select a nonexistent synset."""
        self.assertRaises(
            exc.SynsetNotFound,
            self.__plwn.synset,
            u'not',
            en.PoS.n,
            1,
        )

    def test_synset_by_id_nonexistent(self):
        """Select a nonexistent id."""
        self.assertRaises(
            exc.SynsetNotFound,
            self.__plwn.synset_by_id,
            100,
        )

    def test_lexical_units_nonexistent(self):
        """Select a nonexistent combination of properties."""
        self.assertFalse(tuple(self.__plwn.lexical_units(u'not')))

    def test_lexical_units_by_lemma(self):
        """Select several lexical units sharing a lemma."""
        self.assertEqual(
            frozenset(self.__plwn.lexical_units(u'pies')),
            frozenset((
                self.__plwn.lexical_unit_by_id(11),
                self.__plwn.lexical_unit_by_id(31),
                self.__plwn.lexical_unit_by_id(41),
            )),
        )

    def test_lexical_units_by_pos(self):
        """Select a lexical unit by pos."""
        self.assertEqual(
            frozenset(self.__plwn.lexical_units(pos=en.PoS.v)),
            frozenset((self.__plwn.lexical_unit_by_id(41),)),
        )

    def test_lexical_units_by_combination(self):
        """Select a lexical unit by combination of lemma and pos."""
        self.assertEqual(
            frozenset(
                self.__plwn.lexical_units(lemma=u'pies', pos=en.PoS.n),
            ),
            frozenset((
                self.__plwn.lexical_unit_by_id(11),
                self.__plwn.lexical_unit_by_id(31),
            )),
        )

    def test_lexical_units_one(self):
        """Select one lexical unit using a full combination."""
        self.assertEqual(
            frozenset(
                self.__plwn.lexical_units(
                    lemma=u'kot',
                    pos=en.PoS.n,
                    variant=2,
                )
            ),
            frozenset((self.__plwn.lexical_unit_by_id(22),)),
        )

    def test_lexical_units_all(self):
        """Select all lexical units."""
        self.assertEqual(
            frozenset(self.__plwn.lexical_units()),
            frozenset((
                self.__plwn.lexical_unit_by_id(11),
                self.__plwn.lexical_unit_by_id(21),
                self.__plwn.lexical_unit_by_id(22),
                self.__plwn.lexical_unit_by_id(31),
                self.__plwn.lexical_unit_by_id(41),
            )),
        )

    def test_lexical_unit_any_none(self):
        """Make sure the method does not accept ``None`` as arguments."""
        self.assertRaises(
            exc.LexicalUnitNotFound,
            self.__plwn.lexical_unit,
            None,
            en.PoS.n,
            1,
        )
        self.assertRaises(
            exc.LexicalUnitNotFound,
            self.__plwn.lexical_unit,
            u'kot',
            None,
            1,
        )
        self.assertRaises(
            exc.LexicalUnitNotFound,
            self.__plwn.lexical_unit,
            u'kot',
            en.PoS.n,
            None,
        )

    def test_lexical_unit_one(self):
        """Select a single lexical unit."""
        sel_one = self.__plwn.lexical_unit(
            lemma=u'kot',
            pos=en.PoS.n,
            variant=1,
        )
        sel_id = self.__plwn.lexical_unit_by_id(21)

        self.assertEqual(sel_one, sel_id)
        # As this tests both "lexical_unit" and "lexical_unit_by_id", compare
        # if some of the properties are correct. Don't check all to not
        # duplicate tests; XXX but maybe move all tests to here and remove some
        # external test cases.
        self.assertEqual(sel_one.id, 21)
        self.assertEqual(sel_one.id, sel_id.id)
        self.assertEqual(sel_one.lemma, u'kot')
        self.assertEqual(sel_one.lemma, sel_id.lemma)
        self.assertIs(sel_one.pos, en.PoS.n)
        self.assertEqual(sel_one.pos, sel_id.pos)
        self.assertEqual(sel_one.variant, 1)
        self.assertEqual(sel_one.variant, sel_id.variant)
        self.assertEqual(sel_one.definition, u'kici')
        self.assertEqual(sel_one.definition, sel_id.definition)

    def test_lexical_unit_nonexistent(self):
        """Select a nonexistent lexical unit."""
        self.assertRaises(
            exc.LexicalUnitNotFound,
            self.__plwn.lexical_unit,
            lemma=u'not',
            pos=en.PoS.n,
            variant=1,
        )

    def test_lexical_unit_by_id_nonexistent(self):
        """Select a nonexistent id."""
        self.assertRaises(
            exc.LexicalUnitNotFound,
            self.__plwn.lexical_unit_by_id,
            100,
        )


class SynsetRelationEdgesTest(ut.TestCase):
    """Inspect synset relations in a simple graph.

    ::
            .----.   mero/cz   .----.
            | 4  |<------------| 3  |
            |    |------------>|    |
            '----'   holo/cz   '----'
             ^  |               ^  |
             |  |hipo           |  |hipo
        hiper|  |          hiper|  |
             |  v               |  v
            .----.   mero/cz   .----.
            | 1  |<------------| 2  |
            |    |------------>|    |
            '----'   holo/cz   '----'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiperonimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiponimia',
                aliases=('hipo',),
            ),
            # nd.make_relation_type_node(
            #     kind=en.RelationKind.synset,
            #     name=u'meronimia',
            # ),
            # nd.make_relation_type_node(
            #     kind=en.RelationKind.synset,
            #     name=u'holonimia',
            # ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'część',
                parent=u'meronimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'część',
                parent=u'holonimia',
            ),
            nd.make_synset_node(
                id=1,
                related=(
                    (u'hiperonimia', 4),
                    (
                        RelationInfoBase.format_name(u'holonimia', u'część'),
                        2,
                    ),
                ),
            ),
            nd.make_synset_node(
                id=2,
                related=(
                    (
                        RelationInfoBase.format_name(u'meronimia', u'część'),
                        1,
                    ),
                    (u'hiperonimia', 3),
                ),
            ),
            nd.make_synset_node(
                id=3,
                related=(
                    (u'hiponimia', 2),
                    (
                        RelationInfoBase.format_name(u'meronimia', u'część'),
                        4,
                    ),
                ),
            ),
            nd.make_synset_node(
                id=4,
                related=(
                    (u'hiponimia', 1),
                    (
                        RelationInfoBase.format_name(u'holonimia', u'część'),
                        3,
                    ),
                ),
            ),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=4,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.perf,
            ),
        ))

        try:
            self.__rel_hiper = tuple(self.__plwn.relations_info(
                u'hiperonimia',
                en.RelationKind.synset,
            ))[0]
            self.__rel_hipo = tuple(self.__plwn.relations_info(
                u'hiponimia',
                en.RelationKind.synset,
            ))[0]
            self.__rel_mero = tuple(self.__plwn.relations_info(
                u'meronimia/część',
                en.RelationKind.synset,
            ))[0]
            self.__rel_holo = tuple(self.__plwn.relations_info(
                u'holonimia/część',
                en.RelationKind.synset,
            ))[0]
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_include(self):
        self.assertEqual(
            frozenset(self.__plwn.synset_relation_edges(
                include=(u'hiperonimia', u'meronimia'),
            )),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_mero,
                ),
            )),
        )

    def test_exclude(self):
        self.assertEqual(
            frozenset(self.__plwn.synset_relation_edges(
                exclude=(u'holonimia/część', u'hipo'),
            )),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_mero,
                ),
            )),
        )

    def test_all(self):
        self.assertEqual(
            frozenset(self.__plwn.synset_relation_edges()),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(4),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(2),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(4),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_holo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(2),
                    relation=self.__rel_holo,
                ),
            )),
        )

    def test_combined(self):
        self.assertEqual(
            frozenset(self.__plwn.synset_relation_edges(
                include=(u'meronimia/część', u'hiperonimia'),
                exclude=(u'meronimia', u'holonimia'),
            )),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hiper,
                ),
            )),
        )


class SynsetRelationEdgesWithArtificialTest(ut.TestCase):
    """Get relation edges from a graph.

    Where skipping artificial synsets will produce interesting results.

    ::
             .---.           .---.
             | 3 |           | 4 |
             '---'           '---'
              | ^             ^ |
        hiper | | hipo   hipo | | hiper
              | |             | |
              | '---.----.----' |
              |     | 2a |      |
              '---->'----'<-----'
                     | ^
               hiper | | hipo
                     v |
                    .---.
                    | 1 |
                    '---'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiponimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiperonimia',
            ),
            nd.make_synset_node(
                id=1,
                related=((u'hiponimia', 2),),
            ),
            nd.make_synset_node(
                id=2,
                related=(
                    (u'hiponimia', 3),
                    (u'hiponimia', 4),
                    (u'hiperonimia', 1),
                ),
                is_artificial=True,
            ),
            nd.make_synset_node(
                id=3,
                related=((u'hiperonimia', 2),),
            ),
            nd.make_synset_node(
                id=4,
                related=((u'hiperonimia', 2),),
            ),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=4,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))

        try:
            self.__rel_hiper = tuple(self.__plwn.relations_info(
                u'hiperonimia',
                en.RelationKind.synset,
            ))[0]
            self.__rel_hipo = tuple(self.__plwn.relations_info(
                u'hiponimia',
                en.RelationKind.synset,
            ))[0]
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_all(self):
        self.assertEqual(
            frozenset(
                self.__plwn.synset_relation_edges(skip_artificial=False)
            ),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(2),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(2),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(2),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(4),
                    target=self.__plwn.synset_by_id(2),
                    relation=self.__rel_hiper,
                ),
            )),
        )

    def test_skipping(self):
        self.assertEqual(
            frozenset(
                self.__plwn.synset_relation_edges(skip_artificial=True)
            ),
            frozenset((
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(3),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(1),
                    target=self.__plwn.synset_by_id(4),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(3),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.synset_by_id(4),
                    target=self.__plwn.synset_by_id(1),
                    relation=self.__rel_hiper,
                ),
            )),
        )


class SynsetRelationEdgesWithArtificialLoopTest(ut.TestCase):
    """Checking for inifinite loop while skipping edges in a looping graph.

    ::
        .---. foo .----.
        | 1 |---->| 2a |<--.
        '---'     '----'   |
          ^          |     |
          |foo       |foo  |foo
          |          v     |
        .---. foo .----.   |
        | 4 |<----| 3a |---'
        '---'     '----'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'foo',
            ),
            nd.make_synset_node(id=1, related=((u'foo', 2),)),
            nd.make_synset_node(
                id=2,
                related=((u'foo', 3),),
                is_artificial=True,
            ),
            nd.make_synset_node(
                id=3,
                related=((u'foo', 2), (u'foo', 4)),
                is_artificial=True,
            ),
            nd.make_synset_node(id=4, related=((u'foo', 1),)),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=4,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_all(self):
        syn1 = self.__plwn.synset_by_id(1)
        syn2 = self.__plwn.synset_by_id(2)
        syn3 = self.__plwn.synset_by_id(3)
        syn4 = self.__plwn.synset_by_id(4)
        rel = next(iter(self.__plwn.relations_info(u'foo')), None)
        self.assertEqual(
            frozenset(
                self.__plwn.synset_relation_edges(skip_artificial=False)
            ),
            frozenset((
                RelationEdge(source=syn1, target=syn2, relation=rel),
                RelationEdge(source=syn2, target=syn3, relation=rel),
                RelationEdge(source=syn3, target=syn2, relation=rel),
                RelationEdge(source=syn3, target=syn4, relation=rel),
                RelationEdge(source=syn4, target=syn1, relation=rel),
            )),
        )

    def test_skipping(self):
        syn1 = self.__plwn.synset_by_id(1)
        syn4 = self.__plwn.synset_by_id(4)
        rel = next(iter(self.__plwn.relations_info(u'foo')), None)
        self.assertEqual(
            frozenset(
                self.__plwn.synset_relation_edges(skip_artificial=True)
            ),
            frozenset((
                RelationEdge(source=syn1, target=syn4, relation=rel),
                RelationEdge(source=syn4, target=syn1, relation=rel),
            )),
        )


class LexicalUnitRelationEdgesTest(ut.TestCase):
    """Inspect lexical relations in a simple graph.

    ::
            .----.   mero/cz   .----.
            | 41 |<------------| 31 |
            |    |------------>|    |
            '----'   holo/cz   '----'
             ^  |               ^  |
             |  |hipo           |  |hipo
        hiper|  |          hiper|  |
             |  v               |  v
            .----.   mero/cz   .----.
            | 11 |<------------| 21 |
            |    |------------>|    |
            '----'   holo/cz   '----'

    Relation names are like for synsets, because they don't really matter.
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'hiperonimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'hiponimia',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'część',
                parent=u'holonimia',
                aliases=(u'holo',),
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'część',
                parent=u'meronimia',
            ),
            nd.make_synset_node(id=1),
            nd.make_synset_node(id=2),
            nd.make_synset_node(id=3),
            nd.make_synset_node(id=4),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'holonimia/część', 21), (u'hiperonimia', 41)),
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'bbb',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'meronimia/część', 11), (u'hiperonimia', 31)),
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'ccc',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'meronimia/część', 41), (u'hiponimia', 21)),
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'ddd',
                pos=en.PoS.n,
                variant=1,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
                related=((u'holonimia/część', 31), (u'hiponimia', 11)),
                verb_aspect=en.VerbAspect.perf,
            ),
        ))

        try:
            self.__rel_hiper = tuple(self.__plwn.relations_info(
                u'hiperonimia',
                en.RelationKind.lexical,
            ))[0]
            self.__rel_hipo = tuple(self.__plwn.relations_info(
                u'hiponimia',
                en.RelationKind.lexical,
            ))[0]
            self.__rel_holo = tuple(self.__plwn.relations_info(
                u'holonimia/część',
                en.RelationKind.lexical,
            ))[0]
            self.__rel_mero = tuple(self.__plwn.relations_info(
                u'meronimia/część',
                en.RelationKind.lexical,
            ))[0]
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_include(self):
        self.assertEqual(
            frozenset(self.__plwn.lexical_relation_edges(include=(
                u'hiperonimia',
                u'meronimia/część',
            ))),
            frozenset((
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(11),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(31),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(31),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(11),
                    relation=self.__rel_mero,
                ),
            )),
        )

    def test_exclude(self):
        self.assertEqual(
            frozenset(
                # Also, check aliases
                self.__plwn.lexical_relation_edges(exclude=(u'holo',))
            ),
            frozenset((
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(11),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(31),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(41),
                    target=self.__plwn.lexical_unit_by_id(11),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(31),
                    target=self.__plwn.lexical_unit_by_id(21),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(31),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(11),
                    relation=self.__rel_mero,
                ),
            )),
        )

    def test_all(self):
        self.assertEqual(
            frozenset(self.__plwn.lexical_relation_edges()),
            frozenset((
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(11),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(31),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(31),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(11),
                    relation=self.__rel_mero,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(41),
                    target=self.__plwn.lexical_unit_by_id(11),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(31),
                    target=self.__plwn.lexical_unit_by_id(21),
                    relation=self.__rel_hipo,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(41),
                    target=self.__plwn.lexical_unit_by_id(31),
                    relation=self.__rel_holo,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(11),
                    target=self.__plwn.lexical_unit_by_id(21),
                    relation=self.__rel_holo,
                ),
            )),
        )

    def test_combined(self):
        self.assertEqual(
            frozenset(self.__plwn.lexical_relation_edges(
                include=(u'meronimia', u'hiperonimia'),
                exclude=(u'meronimia', u'holonimia'),
            )),
            frozenset((
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(11),
                    target=self.__plwn.lexical_unit_by_id(41),
                    relation=self.__rel_hiper,
                ),
                RelationEdge(
                    source=self.__plwn.lexical_unit_by_id(21),
                    target=self.__plwn.lexical_unit_by_id(31),
                    relation=self.__rel_hiper,
                ),
            )),
        )
