# coding: utf8
# XXX Relation operators for synsets and lexical units are not explicitly
# tested (only ordering), since they're not very important.
from __future__ import absolute_import, division

import unittest as ut

import six
from plwn.bases import RelationInfoBase
from plwn.readers import nodes as nd
from plwn import exceptions as exc, enums as en


__all__ = (
    'SynsetPropertiesTest',
    'SynsetRelationsTest',
    'SynsetRelationsWithArtificialTest',
    'SynsetRelationsWithArtificialLoopTest',
    'LexicalUnitPropertiesTest',
    'LexicalUnitRelationsTest',
    'ItemOrderingTest',
    'ToDictTest',
)


class SynsetPropertiesTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(id=1, legacy_id=1, definition=u'foobar'),
            nd.make_lexical_unit_node(
                id=11,
                legacy_id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))
        try:
            self.__syn = self.__plwn.synset_by_id(1)
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_id(self):
        self.assertEqual(self.__syn.id, 1)

    def test_definition(self):
        self.assertEqual(self.__syn.definition, u'foobar')

    def test_is_artificial(self):
        self.assertIs(self.__syn.is_artificial, False)


class SynsetRelationsTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'foo',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiperonimia',
                parent=u'foo',
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'hiponimia',
                aliases=(u'hipo',),
            ),
            nd.make_synset_node(
                id=1,
                related=(
                    (
                        RelationInfoBase.format_name(u'foo', u'hiperonimia'),
                        2,
                    ),
                    (u'hiponimia', 3),
                ),
            ),
            nd.make_synset_node(id=2, related=((u'hiponimia', 3),)),
            nd.make_synset_node(id=3),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))
        try:
            self.__syn1 = self.__plwn.synset_by_id(1)
            self.__syn2 = self.__plwn.synset_by_id(2)
            self.__syn3 = self.__plwn.synset_by_id(3)
            self.__rel_hiper = next(iter(self.__plwn.relations_info(
                RelationInfoBase.format_name(u'foo', u'hiperonimia'),
                en.RelationKind.synset,
            )))
            self.__rel_hipo = next(iter(self.__plwn.relations_info(
                u'hiponimia',
                en.RelationKind.synset,
            )))
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_related(self):
        related = tuple(self.__syn1.related(RelationInfoBase.format_name(
            u'foo',
            u'hiperonimia',
        )))
        self.assertEqual(len(related), 1)
        self.assertEqual(related[0], self.__syn2)

    def test_related_alias(self):
        # Try getting relations by alias
        related = tuple(self.__syn1.related(u'hipo'))
        self.assertEqual(len(related), 1)
        self.assertEqual(related[0], self.__syn3)

    def test_empty_related(self):
        self.assertFalse(tuple(self.__syn2.related(u'hiperonimia')))

    def test_nonexistent_relation(self):
        self.assertRaises(
            exc.InvalidRelationTypeException,
            self.__syn1.related,
            u'not',
        )

    def test_all_related(self):
        # Without parameter select all related
        related = frozenset(self.__syn1.related())
        self.assertEqual(len(related), 2)
        self.assertIn(self.__syn2, related)
        self.assertIn(self.__syn3, related)

    def test_tuple_related(self):
        related = frozenset(self.__syn1.related((
            RelationInfoBase.format_name(u'foo', u'hiperonimia'),
            u'hipo',
        )))
        self.assertEqual(len(related), 2)
        self.assertIn(self.__syn2, related)
        self.assertIn(self.__syn3, related)

    def test_related_pairs(self):
        related = frozenset(self.__syn1.related_pairs((
            RelationInfoBase.format_name(u'foo', u'hiperonimia'),
            u'hipo',
        )))
        self.assertEqual(len(related), 2)
        self.assertIn(
            (self.__rel_hiper, self.__syn2),
            related,
        )
        self.assertIn(
            (self.__rel_hipo, self.__syn3),
            related,
        )

    def test_relations(self):
        self.assertEqual(
            frozenset(self.__syn1.relations),
            frozenset((self.__rel_hiper, self.__rel_hipo)),
        )


class SynsetRelationsWithArtificialTest(ut.TestCase):
    """Proper testing of artificial synset skipping requires more complex setup.

    Artificial skipping may skip several nodes in a row.

    ::
             hipo  .---.
          .------->| 3 |
          |        '---'
          |
          |                  hipo  .---.
          |            .---------->| 4 |
          |            |           '---'
          |            |
        .---. hipo  .----.   mero  .---.       .---.
        | 1 |------>| 2a |-------->| 5 |       | 7 |
        '---'       '----'         '---'       '---'
                       |                         ^
                       |     hipo  .----.  hipo  |
                       '---------->| 6a |--------.
                                   '----'        |
                                                 v
                                               .---.
                                               | 8 |
                                               '---'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        # Don't use relation type nodes, as a test for that case.
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(
                id=1,
                related=((u'hiponimia', 3), (u'hiponimia', 2)),
            ),
            nd.make_synset_node(
                id=2,
                related=(
                    (u'hiponimia', 4),
                    (u'hiponimia', 6),
                    (u'meronimia', 5),
                ),
                is_artificial=True,
            ),
            nd.make_synset_node(id=3),
            nd.make_synset_node(id=4),
            nd.make_synset_node(id=5),
            nd.make_synset_node(
                id=6,
                related=((u'hiponimia', 7), (u'hiponimia', 8)),
                is_artificial=True,
            ),
            nd.make_synset_node(id=7),
            nd.make_synset_node(id=8),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=4,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=51,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=5,
                synset=5,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=61,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=6,
                synset=6,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=71,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=7,
                synset=7,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=81,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=8,
                synset=8,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))

        try:
            self.__syn1 = self.__plwn.synset_by_id(1)
            self.__rel_hipo = next(iter(self.__plwn.relations_info(
                u'hiponimia',
                en.RelationKind.synset,
            )))
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_related_noskip(self):
        self.assertEqual(
            frozenset(
                self.__syn1.related(u'hiponimia', skip_artificial=False)
            ),
            frozenset((
                self.__plwn.synset_by_id(2),
                self.__plwn.synset_by_id(3),
            )),
        )

    def test_related_skip(self):
        self.assertEqual(
            frozenset(self.__syn1.related(u'hiponimia', skip_artificial=True)),
            frozenset((
                self.__plwn.synset_by_id(3),
                self.__plwn.synset_by_id(4),
                self.__plwn.synset_by_id(7),
                self.__plwn.synset_by_id(8),
            )),
        )

    def test_related_pairs_noskip(self):
        self.assertEqual(
            frozenset(
                self.__syn1.related_pairs(u'hiponimia', skip_artificial=False)
            ),
            frozenset((
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(2),
                ),
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(3),
                ),
            )),
        )

    def test_related_pairs_skip(self):
        self.assertEqual(
            frozenset(self.__syn1.related_pairs(
                u'hiponimia',
                skip_artificial=True,
            )),
            frozenset((
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(3),
                ),
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(4),
                ),
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(7),
                ),
                (
                    self.__rel_hipo,
                    self.__plwn.synset_by_id(8),
                ),
            )),
        )


class SynsetRelationsWithArtificialLoopTest(ut.TestCase):
    """Test for an infinite loop.

    Where skipping artificial synsets would result in an infinite loop.

    ::
                         foo
                     .----------.
                     |          |
                     v          |
        .---. foo .----. foo .----. foo .---.
        | 1 |---->| 2a |---->| 3a |---->| 4 |
        '---'     '----'     '----'     '---'
    """

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.synset,
                name=u'foo',
            ),
            nd.make_synset_node(id=1, related=((u'foo', 2),)),
            nd.make_synset_node(
                id=2,
                related=((u'foo', 3),),
                is_artificial=True,
            ),
            nd.make_synset_node(
                id=3,
                related=((u'foo', 4), (u'foo', 2)),
                is_artificial=True,
            ),
            nd.make_synset_node(id=4),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=3,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=41,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=4,
                synset=4,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_related_noskip(self):
        syn1 = self.__plwn.synset_by_id(1)
        rel1 = frozenset(syn1.related(u'foo', skip_artificial=False))
        self.assertEqual(
            rel1,
            frozenset((self.__plwn.synset_by_id(2),)),
        )

    def test_related_skip(self):
        syn1 = self.__plwn.synset_by_id(1)
        rel1 = frozenset(syn1.related(u'foo', skip_artificial=True))
        self.assertEqual(
            rel1,
            frozenset((self.__plwn.synset_by_id(4),)),
        )

    def test_related_pairs_noskip(self):
        syn1 = self.__plwn.synset_by_id(1)
        rel1 = frozenset(syn1.related_pairs(u'foo', skip_artificial=False))
        foo = next(iter(self.__plwn.relations_info(u'foo')), None)
        self.assertEqual(
            rel1,
            frozenset(((foo, self.__plwn.synset_by_id(2)),)),
        )

    def test_related_pairs_skip(self):
        syn1 = self.__plwn.synset_by_id(1)
        rel1 = frozenset(syn1.related_pairs(u'foo', skip_artificial=True))
        foo = next(iter(self.__plwn.relations_info(u'foo')), None)
        self.assertEqual(
            rel1,
            frozenset(((foo, self.__plwn.synset_by_id(4)),)),
        )


class LexicalUnitPropertiesTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(id=1),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                definition=u'scary noise',
                usage_notes=(u'K', u'B'),
                external_links=(u'http://foo.bar',),
                examples=(u'aaaand',),
                examples_sources=(u'me',),
                domain=en.Domain.cdel,
                verb_aspect=en.VerbAspect.two_aspect,
                is_emotional=True,
                emotion_markedness=en.EmotionMarkedness.ambiguous,
                emotion_names=(en.EmotionName.joy, en.EmotionName.trust),
                emotion_valuations=(
                    en.EmotionValuation.error,
                    en.EmotionValuation.good,
                ),
                emotion_example_1=u'bim',
                emotion_example_2=u'bam',
            ),
        ))
        try:
            self.__syn = self.__plwn.synset_by_id(1)
            self.__lex = self.__plwn.lexical_unit_by_id(11)
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_id(self):
        self.assertEqual(self.__lex.id, 11)

    def test_lemma(self):
        self.assertEqual(self.__lex.lemma, u'aaa')

    def test_pos(self):
        self.assertEqual(self.__lex.pos, en.PoS.n)

    def test_variant(self):
        self.assertEqual(self.__lex.variant, 1)

    def test_synset(self):
        self.assertEqual(self.__lex.synset, self.__syn)

    def test_definition(self):
        self.assertEqual(self.__lex.definition, u'scary noise')

    def test_usage_notes(self):
        self.assertEqual(self.__lex.usage_notes, (u'K', u'B'))

    def test_external_links(self):
        self.assertEqual(self.__lex.external_links, (u'http://foo.bar',))

    def test_examples(self):
        self.assertEqual(self.__lex.sense_examples, (u'aaaand',))

    def test_examples_sources(self):
        self.assertEqual(self.__lex.sense_examples_sources, (u'me',))

    def test_domain(self):
        self.assertEqual(self.__lex.domain, en.Domain.cdel)

    def test_verb_aspect(self):
        self.assertEqual(self.__lex.verb_aspect, en.VerbAspect.two)

    def test_is_emotional(self):
        self.assertTrue(self.__lex.is_emotional)

    def test_emotion_markedness(self):
        self.assertIs(
            self.__lex.emotion_markedness,
            en.EmotionMarkedness.ambiguous,
        )

    def test_emotion_names(self):
        self.assertEqual(
            self.__lex.emotion_names,
            (en.EmotionName.joy, en.EmotionName.trust),
        )

    def test_emotion_valuations(self):
        self.assertEqual(
            self.__lex.emotion_valuations,
            (en.EmotionValuation.error, en.EmotionValuation.good),
        )

    def test_emotion_example(self):
        self.assertEqual(self.__lex.emotion_example, u'bim')

    def test_emotion_example_secondary(self):
        self.assertEqual(self.__lex.emotion_example_secondary, u'bam')


class LexicalUnitRelationsTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'deminutywność',
                aliases=(u'demi',),
            ),
            nd.make_relation_type_node(
                kind=en.RelationKind.lexical,
                name=u'mpar',
                parent=u'syn',
            ),
            nd.make_synset_node(id=1),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
                related=(
                    (u'deminutywność', 12),
                    (RelationInfoBase.format_name(u'syn', u'mpar'), 13),
                ),
            ),
            nd.make_lexical_unit_node(
                id=12,
                lemma=u'bbb',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=2,
                domain=en.Domain.bhp,
                related=((u'deminutywność', 13),),
            ),
            nd.make_lexical_unit_node(
                id=13,
                lemma=u'ccc',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=3,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.two,
            ),
        ))
        try:
            self.__lex11 = self.__plwn.lexical_unit_by_id(11)
            self.__lex12 = self.__plwn.lexical_unit_by_id(12)
            self.__lex13 = self.__plwn.lexical_unit_by_id(13)
            self.__rel_mpar = next(iter(self.__plwn.relations_info(
                RelationInfoBase.format_name(u'syn', u'mpar'),
                en.RelationKind.lexical,
            )))
            self.__rel_demi = next(iter(self.__plwn.relations_info(
                u'deminutywność',
                en.RelationKind.lexical,
            )))
        except BaseException:
            self.__plwn.close()
            raise

    def tearDown(self):
        self.__plwn.close()

    def test_related(self):
        related = tuple(self.__lex11.related(RelationInfoBase.format_name(
            u'syn',
            u'mpar',
        )))
        self.assertEqual(len(related), 1)
        self.assertEqual(related[0], self.__lex13)

    def test_related_alias(self):
        related = tuple(self.__lex12.related(u'demi'))
        self.assertEqual(len(related), 1)
        self.assertEqual(related[0], self.__lex13)

    def test_empty_related(self):
        self.assertFalse(tuple(self.__lex12.related(
            RelationInfoBase.format_name(u'syn', u'mpar'),
        )))

    def test_nonexistent_relation(self):
        self.assertRaises(
            exc.InvalidRelationTypeException,
            self.__lex11.related,
            u'not',
        )

    def test_all_related(self):
        # Without parameter select all related
        related = frozenset(self.__lex11.related())
        self.assertEqual(len(related), 2)
        self.assertIn(self.__lex12, related)
        self.assertIn(self.__lex13, related)

    def test_tuple_related(self):
        related = frozenset(self.__lex11.related((
            self.__rel_demi,
            self.__rel_mpar,
        )))
        self.assertEqual(len(related), 2)
        self.assertIn(self.__lex12, related)
        self.assertIn(self.__lex13, related)

    def test_related_pairs(self):
        related = frozenset(self.__lex11.related_pairs())
        self.assertEqual(len(related), 2)
        self.assertIn(
            (self.__rel_demi, self.__lex12),
            related,
        )
        self.assertIn(
            (self.__rel_mpar, self.__lex13),
            related,
        )

    def test_relations(self):
        self.assertEqual(
            frozenset(self.__lex11.relations),
            frozenset((self.__rel_demi, self.__rel_mpar)),
        )


class ItemOrderingTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(id=1),
            nd.make_synset_node(id=2),
            nd.make_synset_node(id=3),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.n,
                variant=1,
                synset=1,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'bbb',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=22,
                lemma=u'ąąą',
                pos=en.PoS.n,
                variant=2,
                synset=2,
                unit_index=2,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'ąąą',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=1,
                domain=en.Domain.bhp,
                verb_aspect=en.VerbAspect.two,
            ),
        ))

    def tearDown(self):
        self.__plwn.close()

    def test_synsets_order(self):
        syn1 = self.__plwn.synset_by_id(1)
        syn2 = self.__plwn.synset_by_id(2)
        syn3 = self.__plwn.synset_by_id(3)

        list_ = [syn3, syn2, syn1]
        list_.sort()
        self.assertEqual(list_, [syn1, syn3, syn2])

    def test_lexical_units_order(self):
        lex11 = self.__plwn.lexical_unit_by_id(11)
        lex21 = self.__plwn.lexical_unit_by_id(21)
        lex22 = self.__plwn.lexical_unit_by_id(22)
        lex31 = self.__plwn.lexical_unit_by_id(31)

        list_ = [lex31, lex22, lex21, lex11]
        list_.sort()
        self.assertEqual(list_, [lex11, lex31, lex22, lex21])


class ToDictTest(ut.TestCase):

    _PLWNClass = None  # Override in subclass

    def setUp(self):
        self.maxDiff = None
        # Try not using the relation type nodes here
        self.__plwn = self._PLWNClass.from_reader((
            nd.make_synset_node(
                id=1,
                definition=u'foo',
                related=((u'hiperonimia', 2), (u'hiponimia', 3)),
            ),
            nd.make_synset_node(id=2),
            nd.make_synset_node(id=3),
            nd.make_lexical_unit_node(
                id=11,
                lemma=u'aaa',
                pos=en.PoS.v,
                variant=1,
                synset=1,
                unit_index=0,
                definition=u'bar',
                usage_notes=(u'baz', u'ban'),
                external_links=(u'http://a.com',),
                examples=(u'nothing',),
                examples_sources=(u'none',),
                domain=en.Domain.rz,
                related=((u'derywacyjność', 21),),
                verb_aspect=en.VerbAspect.pred,
                emotion_markedness=en.EmotionMarkedness.strong_negative,
                emotion_names=(en.EmotionName.surprise,),
                emotion_valuations=[
                    en.EmotionValuation.error,
                    en.EmotionValuation.ugliness,
                ],
                emotion_example_1=u'Bad thing.',
            ),
            nd.make_lexical_unit_node(
                id=21,
                lemma=u'bbb',
                pos=en.PoS.n,
                variant=1,
                synset=2,
                unit_index=1,
                domain=en.Domain.bhp,
            ),
            nd.make_lexical_unit_node(
                id=31,
                lemma=u'ccc',
                pos=en.PoS.n,
                variant=1,
                synset=3,
                unit_index=0,
                domain=en.Domain.bhp,
            ),
        ))
        self.__lex11_dict = {
            u'id': 11,
            u'lemma': u'aaa',
            u'pos': u'czasownik',
            u'variant': 1,
            u'synset': 1,
            u'definition': u'bar',
            u'usage_notes': (u'baz', u'ban'),
            u'external_links': (u'http://a.com',),
            u'sense_examples': (u'nothing',),
            u'sense_examples_sources': (u'none',),
            u'domain': en.Domain.rz.value,
            u'verb_aspect': en.VerbAspect.predicative.value,
            u'emotion_markedness': en.EmotionMarkedness.minus_m.value,
            u'emotion_names': (en.EmotionName.surprise.value,),
            u'emotion_valuations': (
                en.EmotionValuation.error.value,
                en.EmotionValuation.ugliness.value,
            ),
            u'emotion_example': u'Bad thing.',
            u'emotion_example_secondary': None,
            u'str': six.text_type(self.__plwn.lexical_unit_by_id(11)),
        }
        self.__lex11_dict_with_rel = {
            u'related': {
                u'derywacyjność': (
                    (
                        21,
                        six.text_type(
                            self.__plwn.lexical_unit_by_id(21)
                        ),
                    ),
                ),
            },
        }
        self.__lex11_dict_with_rel.update(self.__lex11_dict)

    def tearDown(self):
        self.__plwn.close()

    def test_lexunit_dict(self):
        lex11 = self.__plwn.lexical_unit_by_id(11)
        self.assertEqual(
            lex11.to_dict(include_related=False),
            self.__lex11_dict,
        )
        self.assertEqual(lex11.to_dict(), self.__lex11_dict_with_rel)

    def test_synset_dict(self):
        syn1 = self.__plwn.synset_by_id(1)

        self.assertEqual(
            syn1.to_dict(include_related=False, include_units_data=False),
            {
                u'id': 1,
                u'str': six.text_type(syn1),
                u'definition': u'foo',
                u'units': (
                    (
                        11,
                        six.text_type(
                            self.__plwn.lexical_unit_by_id(11)
                        ),
                    ),
                ),
                u'is_artificial': False,
            },
        )
        self.assertEqual(
            syn1.to_dict(include_related=False),
            {
                u'id': 1,
                u'str': six.text_type(syn1),
                u'definition': u'foo',
                u'units': (self.__lex11_dict,),
                u'is_artificial': False,
            },
        )
        self.assertEqual(
            syn1.to_dict(),
            {
                u'id': 1,
                u'str': six.text_type(syn1),
                u'definition': u'foo',
                u'units': (self.__lex11_dict_with_rel,),
                u'related': {
                    u'hiperonimia': (
                        (2, six.text_type(self.__plwn.synset_by_id(2))),
                    ),
                    u'hiponimia': (
                        (3, six.text_type(self.__plwn.synset_by_id(3))),
                    ),
                },
                u'is_artificial': False,
            },
        )
